# $Id: user.py,v 1.69 2010/11/15 04:23:25 irees Exp $
import re
import os

import emen2.db.config
g = emen2.db.config.g()
from emen2.web.view import View



@View.register
class User(View):


	@View.add_matcher("^/user/(?P<username>[\w\- ]+)/$")
	def init(self, username=None):
		self.template = "/pages/user"
		self.user = self.db.getuser(username, getgroups=True, filt=False)
		self.username = self.user.username

		displaynames = {}
		for i in self.db.getuser([self.user.record]):
			displaynames[i.username] = i.displayname

		self.admin = self.db.checkadmin()
		try:
			self.admin = user.userrec.writable()
		except:
			pass

		self.ctxt["admin"] = self.admin
		self.title = "User: %s (%s)"%(self.user.displayname, username)
		self.ctxt["user"] = self.user



	@View.add_matcher("^/user/(?P<username>[\w\- ]+)/edit/$")
	def edit(self, username=None, notify=[], **kwargs):
		self.init(username=username)
		self.template = "/pages/user.edit"

		if self.db.checkcontext()[0] != self.username and not self.admin:
			raise emen2.db.exceptions.SecurityError, "You may only edit your own user page"

		self.title = "Profile Editor"
		self.ctxt["admin"] = self.admin



	@View.add_matcher("^/user/(?P<username>[\w\- ]+)/save/$")
	def save(self, username=None, action=None, notify=[], **kwargs):
		self.edit(username=username, **kwargs)
		kw_userrec = kwargs.pop('userrec',None)
		kw_user = kwargs.pop('user', None)
		self.action_save(kw_userrec, kw_user, kwargs)
		self.ctxt["notify"].append("Changes Saved")
		self.ctxt["user"] = self.user


	#@write
	def action_save(self, kw_userrec, kw_user, kwargs):
		# for now, only allow edit of either user or userrec
		if kw_userrec:
			orec = self.db.getrecord(self.user.record)
			for k,v in kw_userrec.items():
				orec[k] = v

			# of course clever people can edit the records directly,
			# but for convenience we'll require some fields to be filled here:
			for param in ["name_first","name_last"]:
				if orec.get(param) == None:
					raise Exception, "Required item %s empty"%param

			self.db.putrecord(orec)

		elif kw_user:
			email = kw_user.get('email')
			disabled = kw_user.get('disabled')
			privacy = kw_user.get('privacy')

			if privacy != None:
				self.db.setprivacy(privacy, username=self.username)

			if disabled != None:
				disabled = int(disabled)
				if disabled:
					self.db.disableuser(self.username)
				else:
					self.db.enableuser(self.username)


		self.user = self.db.getuser(self.username, getgroups=True)






class Users(View):
	__metaclass__ = View.register_view
	__matcher__ = r'^/users/$'

	def init(self, q=None):
		self.template = "/pages/users"
		self.title = "User Directory"

		if q:
			users = self.db.finduser(q)
		else:
			users = self.db.getuser(self.db.getusernames())

		if not users:
			self.template = "/pages/page"
			self.ctxt["content"] = """No users found, or insufficient permissions to view user roster."""
			return

		self.ctxt["users"] = users
		self.ctxt["q"] = q or ''





class AdminUsers(View):
	__metaclass__ = View.register_view
	__matcher__ = r'^/users/admin/$'

	def init(self, sortby="name_last", reverse=0, q=None, **kwargs):

		self.template="/pages/users.admin"
		self.update_context(args=kwargs, title="User Management", q=q)

		# u = [i.username for i in self.db.finduser(query=unicode(q))] #[i[0] for i in ]

		try:
			reverse = int(reverse)
		except:
			reverse = 0

		if q:
			users = self.db.finduser(q)
		else:
			usernames = self.db.getusernames()
			users = self.db.getuser(usernames)

		admin = self.db.checkadmin()
		self.ctxt.update({"users":users, "admin": admin, "sortby":sortby, "reverse":reverse})








# moved from newuser.py
REQUIRED_USERREC_PARAMS = set(["name_first","name_last"])



# class ConfirmEmail(View):
# 	__metaclass__ = View.register_view
# 	__matcher__ = '^/users/new/confirm/$'
# 
# 	def init(self, *_, **__):
# 		self.template='/admin/newuser.confirmemail'





@View.register
class NewUser(emen2.web.view.View):

	#@write
	@View.add_matcher("^/users/new/save/$")
	def save(self, **kwargs):
		self.title = 'New User Application'
		self.ctxt['kwargs'] = kwargs

		msg = ''
		errmsg = ''

		validator = emen2.db.validators.InputValidator()
		for i in REQUIRED_USERREC_PARAMS:
			validator.add_predicate(i)
		#validator.add_predicate('email', lambda x: '@' in x)

		self.ctxt["invalid_fields"] = validator.check_dictionary(kwargs)
		if self.ctxt["invalid_fields"]:
			self.template = '/pages/users.new'
			return

		username = ''.join([kwargs.get("name_first"),kwargs.get("name_last")]).lower()
		rd = self.db.newuser(username=username, password=kwargs.get("password"), email=kwargs.get("email"))

		signupfields=["name_first","name_middle","name_last","email","institution","department","address_street","address_city", \
			"address_state","address_zipcode","country","website","phone_voice","phone_fax","comments"]

		for k in signupfields:
			rd.signupinfo[k] = kwargs.get(k,None)

		try:
			self.db.adduser(rd)
		except Exception, inst:
			self.template = "/errors/error"
			self.ctxt["errmsg"] = "Problem creating user account: <p>%s</p>"%inst
			return

		self.template = "/pages/page"
		self.ctxt['content'] = '''
		<h1>New User Request</h1>

		<p>Your request for a new account is being processed. You will be notified via email when it is approved.</p>

		<p>Username: %s</p>
		<p>Email: %s</p>

		'''%(rd.username,rd.email)


	@View.add_matcher("^/users/new/$")
	def init(self, **kwargs):
		self.template = '/pages/users.new'
		self.title = 'New User Application'
		self.ctxt['kwargs'] = kwargs
		self.ctxt["invalid_fields"] = []




# class ViewNewUser(emen2.web.view.View):
# 	__metaclass__ = emen2.web.view.View.register_view
# 	__matcher__ = '^/viewnewuser/(?P<username>\w+)/$'
#
# 	def init(self, username=None, **kwargs):
# 		self.template='/admin/viewnewuser'
# 		self.__user = dict(self.db.getqueueduser(username))
# 		self.__user['name'] = ' '.join(self.__user['name'])
# 		del self.__user['password']
# 		self.ctxt['user'] = self.__user




class UserQueue(emen2.web.view.View):
	__metaclass__ = emen2.web.view.View.register_view
	__matcher__ = ['^/users/queue/$','^/users/new/(?P<username>\w+)/(?P<action>\w+)/$']


	def init(self, action=None, username=None, **kwargs):
		self.template='/pages/users.queue'

		admin_queue = {}
		for i in self.db.getuserqueue():
			admin_queue[i]=self.db.getqueueduser(i)

		self.ctxt.update({"admin_queue":admin_queue})


__version__ = "$Revision: 1.69 $".split(":")[1][:-1].strip()
